@@ -25,7 +25,8 @@ GitHub. See Python Software Foundation License and BSD licenses for these.
 
 #include "../headers/portable.h"
 
-void coliter_setup(coliter_t *self, parser_t *parser, int i, int start) {
+void coliter_setup(coliter_t *self, parser_t *parser, int64_t i,
+                   int64_t start) {
     // column i, starting at 0
     self->words = parser->words;
     self->col = i;
@@ -91,11 +92,9 @@ void parser_set_default_options(parser_t *self) {
     self->skipinitialspace = 0;
     self->quoting = QUOTE_MINIMAL;
     self->allow_embedded_newline = 1;
-    self->strict = 0;
 
     self->expected_fields = -1;
-    self->error_bad_lines = 0;
-    self->warn_bad_lines = 0;
+    self->on_bad_lines = ERROR;
 
     self->commentchar = '#';
     self->thousands = '\0';
@@ -159,7 +158,7 @@ int parser_init(parser_t *self) {
     self->warn_msg = NULL;
 
     // token stream
-    self->stream = (char *)malloc(STREAM_INIT_SIZE * sizeof(char));
+    self->stream = malloc(STREAM_INIT_SIZE * sizeof(char));
     if (self->stream == NULL) {
         parser_cleanup(self);
         return PARSER_OUT_OF_MEMORY;
@@ -170,16 +169,16 @@ int parser_init(parser_t *self) {
     // word pointers and metadata
     sz = STREAM_INIT_SIZE / 10;
     sz = sz ? sz : 1;
-    self->words = (char **)malloc(sz * sizeof(char *));
-    self->word_starts = (int64_t *)malloc(sz * sizeof(int64_t));
+    self->words = malloc(sz * sizeof(char *));
+    self->word_starts = malloc(sz * sizeof(int64_t));
     self->max_words_cap = sz;
     self->words_cap = sz;
     self->words_len = 0;
 
     // line pointers and metadata
-    self->line_start = (int64_t *)malloc(sz * sizeof(int64_t));
+    self->line_start = malloc(sz * sizeof(int64_t));
 
-    self->line_fields = (int64_t *)malloc(sz * sizeof(int64_t));
+    self->line_fields = malloc(sz * sizeof(int64_t));
 
     self->lines_cap = sz;
     self->lines = 0;
@@ -345,7 +344,7 @@ static int push_char(parser_t *self, char c) {
              "self->stream_cap(%d)\n",
              self->stream_len, self->stream_cap))
         int64_t bufsize = 100;
-        self->error_msg = (char *)malloc(bufsize);
+        self->error_msg = malloc(bufsize);
         snprintf(self->error_msg, bufsize,
                  "Buffer overflow caught - possible malformed input file.\n");
         return PARSER_OUT_OF_MEMORY;
@@ -362,7 +361,7 @@ int PANDAS_INLINE end_field(parser_t *self) {
              "self->words_cap(%zu)\n",
              self->words_len, self->words_cap))
         int64_t bufsize = 100;
-        self->error_msg = (char *)malloc(bufsize);
+        self->error_msg = malloc(bufsize);
         snprintf(self->error_msg, bufsize,
                  "Buffer overflow caught - possible malformed input file.\n");
         return PARSER_OUT_OF_MEMORY;
@@ -398,7 +397,7 @@ static void append_warning(parser_t *self, const char *msg) {
     void *newptr;
 
     if (self->warn_msg == NULL) {
-        self->warn_msg = (char *)malloc(length + 1);
+        self->warn_msg = malloc(length + 1);
         snprintf(self->warn_msg, length + 1, "%s", msg);
     } else {
         ex_length = strlen(self->warn_msg);
@@ -413,7 +412,7 @@ static void append_warning(parser_t *self, const char *msg) {
 static int end_line(parser_t *self) {
     char *msg;
     int64_t fields;
-    int ex_fields = self->expected_fields;
+    int64_t ex_fields = self->expected_fields;
     int64_t bufsize = 100;  // for error or warning messages
 
     fields = self->line_fields[self->lines];
@@ -447,7 +446,7 @@ static int end_line(parser_t *self) {
     }
 
     if (!(self->lines <= self->header_end + 1) &&
-        (self->expected_fields < 0 && fields > ex_fields) && !(self->usecols)) {
+        (fields > ex_fields) && !(self->usecols)) {
         // increment file line count
         self->file_lines++;
 
@@ -458,24 +457,24 @@ static int end_line(parser_t *self) {
         self->line_fields[self->lines] = 0;
 
         // file_lines is now the actual file line number (starting at 1)
-        if (self->error_bad_lines) {
-            self->error_msg = (char *)malloc(bufsize);
+        if (self->on_bad_lines == ERROR) {
+            self->error_msg = malloc(bufsize);
             snprintf(self->error_msg, bufsize,
-                    "Expected %d fields in line %lld, saw %lld\n",
-                    ex_fields, (long long)self->file_lines, (long long)fields);
+                    "Expected %" PRId64 " fields in line %" PRIu64 ", saw %"
+                    PRId64 "\n", ex_fields, self->file_lines, fields);
 
             TRACE(("Error at line %d, %d fields\n", self->file_lines, fields));
 
             return -1;
         } else {
             // simply skip bad lines
-            if (self->warn_bad_lines) {
+            if (self->on_bad_lines == WARN) {
                 // pass up error message
-                msg = (char *)malloc(bufsize);
+                msg = malloc(bufsize);
                 snprintf(msg, bufsize,
-                        "Skipping line %lld: expected %d fields, saw %lld\n",
-                         (long long)self->file_lines, ex_fields,
-                         (long long)fields);
+                        "Skipping line %" PRIu64 ": expected %" PRId64
+                        " fields, saw %" PRId64 "\n",
+                        self->file_lines, ex_fields, fields);
                 append_warning(self, msg);
                 free(msg);
             }
@@ -487,7 +486,7 @@ static int end_line(parser_t *self) {
             // might overrun the buffer when closing fields
             if (make_stream_space(self, ex_fields - fields) < 0) {
                 int64_t bufsize = 100;
-                self->error_msg = (char *)malloc(bufsize);
+                self->error_msg = malloc(bufsize);
                 snprintf(self->error_msg, bufsize, "out of memory");
                 return -1;
             }
@@ -508,7 +507,7 @@ static int end_line(parser_t *self) {
                 "end_line: ERROR!!! self->lines(%zu) >= self->lines_cap(%zu)\n",
                 self->lines, self->lines_cap))
             int64_t bufsize = 100;
-            self->error_msg = (char *)malloc(bufsize);
+            self->error_msg = malloc(bufsize);
             snprintf(self->error_msg, bufsize,
                      "Buffer overflow caught - "
                      "possible malformed input file.\n");
@@ -555,13 +554,15 @@ int parser_set_skipfirstnrows(parser_t *self, int64_t nrows) {
     return 0;
 }
 
-static int parser_buffer_bytes(parser_t *self, size_t nbytes) {
+static int parser_buffer_bytes(parser_t *self, size_t nbytes,
+                               const char *encoding_errors) {
     int status;
     size_t bytes_read;
 
     status = 0;
     self->datapos = 0;
-    self->data = self->cb_io(self->source, nbytes, &bytes_read, &status);
+    self->data = self->cb_io(self->source, nbytes, &bytes_read, &status,
+                             encoding_errors);
     TRACE((
         "parser_buffer_bytes self->cb_io: nbytes=%zu, datalen: %d, status=%d\n",
         nbytes, bytes_read, status));
@@ -569,7 +570,7 @@ static int parser_buffer_bytes(parser_t *self, size_t nbytes) {
 
     if (status != REACHED_EOF && self->data == NULL) {
         int64_t bufsize = 200;
-        self->error_msg = (char *)malloc(bufsize);
+        self->error_msg = malloc(bufsize);
 
         if (status == CALLING_READ_FAILED) {
             snprintf(self->error_msg, bufsize,
@@ -600,7 +601,7 @@ static int parser_buffer_bytes(parser_t *self, size_t nbytes) {
         TRACE(("PUSH_CHAR: ERROR!!! slen(%d) >= stream_cap(%d)\n", slen,      \
                self->stream_cap))                                             \
         int64_t bufsize = 100;                                                \
-        self->error_msg = (char *)malloc(bufsize);                            \
+        self->error_msg = malloc(bufsize);                                    \
         snprintf(self->error_msg, bufsize,                                    \
                  "Buffer overflow caught - possible malformed input file.\n");\
         return PARSER_OUT_OF_MEMORY;                                          \
@@ -648,7 +649,7 @@ static int parser_buffer_bytes(parser_t *self, size_t nbytes) {
 #define END_LINE() END_LINE_STATE(START_RECORD)
 
 #define IS_TERMINATOR(c)                            \
-    (c == line_terminator)
+    (c == lineterminator)
 
 #define IS_QUOTE(c) ((c == self->quotechar && self->quoting != QUOTE_NONE))
 
@@ -717,7 +718,7 @@ int tokenize_bytes(parser_t *self,
     char *stream;
     char *buf = self->data + self->datapos;
 
-    const char line_terminator = (self->lineterminator == '\0') ?
+    const char lineterminator = (self->lineterminator == '\0') ?
             '\n' : self->lineterminator;
 
     // 1000 is something that couldn't fit in "char"
@@ -730,7 +731,7 @@ int tokenize_bytes(parser_t *self,
 
     if (make_stream_space(self, self->datalen - self->datapos) < 0) {
         int64_t bufsize = 100;
-        self->error_msg = (char *)malloc(bufsize);
+        self->error_msg = malloc(bufsize);
         snprintf(self->error_msg, bufsize, "out of memory");
         return -1;
     }
@@ -1032,15 +1033,9 @@ int tokenize_bytes(parser_t *self,
                 } else if (IS_CARRIAGE(c)) {
                     END_FIELD();
                     self->state = EAT_CRNL;
-                } else if (!self->strict) {
+                } else {
                     PUSH_CHAR(c);
                     self->state = IN_FIELD;
-                } else {
-                    int64_t bufsize = 100;
-                    self->error_msg = (char *)malloc(bufsize);
-                    snprintf(self->error_msg, bufsize,
-                            "delimiter expected after quote in quote");
-                    goto parsingerror;
                 }
                 break;
 
@@ -1150,8 +1145,8 @@ static int parser_handle_eof(parser_t *self) {
         case IN_QUOTED_FIELD:
             self->error_msg = (char *)malloc(bufsize);
             snprintf(self->error_msg, bufsize,
-                    "EOF inside string starting at row %lld",
-                    (long long)self->file_lines);
+                    "EOF inside string starting at row %" PRIu64,
+                    self->file_lines);
             return -1;
 
         case ESCAPED_CHAR:
@@ -1203,7 +1198,7 @@ int parser_consume_rows(parser_t *self, size_t nrows) {
 
     /* move stream, only if something to move */
     if (char_count < self->stream_len) {
-        memmove((void *)self->stream, (void *)(self->stream + char_count),
+        memmove(self->stream, (self->stream + char_count),
                 self->stream_len - char_count);
     }
     /* buffer counts */
@@ -1269,20 +1264,16 @@ int parser_trim_buffers(parser_t *self) {
     new_cap = _next_pow2(self->words_len) + 1;
     if (new_cap < self->words_cap) {
         TRACE(("parser_trim_buffers: new_cap < self->words_cap\n"));
-        newptr = realloc((void *)self->words, new_cap * sizeof(char *));
-        if (newptr == NULL) {
+        self->words = realloc(self->words, new_cap * sizeof(char *));
+        if (self->words == NULL) {
             return PARSER_OUT_OF_MEMORY;
-        } else {
-            self->words = (char **)newptr;
         }
-        newptr = realloc((void *)self->word_starts,
-                              new_cap * sizeof(int64_t));
-        if (newptr == NULL) {
+        self->word_starts = realloc(self->word_starts,
+                                    new_cap * sizeof(int64_t));
+        if (self->word_starts == NULL) {
             return PARSER_OUT_OF_MEMORY;
-        } else {
-            self->word_starts = (int64_t *)newptr;
-            self->words_cap = new_cap;
         }
+        self->words_cap = new_cap;
     }
 
     /* trim stream */
@@ -1295,7 +1286,7 @@ int parser_trim_buffers(parser_t *self) {
         TRACE(
             ("parser_trim_buffers: new_cap < self->stream_cap, calling "
              "realloc\n"));
-        newptr = realloc((void *)self->stream, new_cap);
+        newptr = realloc(self->stream, new_cap);
         if (newptr == NULL) {
             return PARSER_OUT_OF_MEMORY;
         } else {
@@ -1321,19 +1312,19 @@ int parser_trim_buffers(parser_t *self) {
     new_cap = _next_pow2(self->lines) + 1;
     if (new_cap < self->lines_cap) {
         TRACE(("parser_trim_buffers: new_cap < self->lines_cap\n"));
-        newptr = realloc((void *)self->line_start,
+        newptr = realloc(self->line_start,
                               new_cap * sizeof(int64_t));
         if (newptr == NULL) {
             return PARSER_OUT_OF_MEMORY;
         } else {
-            self->line_start = (int64_t *)newptr;
+            self->line_start = newptr;
         }
-        newptr = realloc((void *)self->line_fields,
+        newptr = realloc(self->line_fields,
                               new_cap * sizeof(int64_t));
         if (newptr == NULL) {
             return PARSER_OUT_OF_MEMORY;
         } else {
-            self->line_fields = (int64_t *)newptr;
+            self->line_fields = newptr;
             self->lines_cap = new_cap;
         }
     }
@@ -1346,7 +1337,8 @@ int parser_trim_buffers(parser_t *self) {
   all : tokenize all the data vs. certain number of rows
  */
 
-int _tokenize_helper(parser_t *self, size_t nrows, int all) {
+int _tokenize_helper(parser_t *self, size_t nrows, int all,
+                     const char *encoding_errors) {
     int status = 0;
     uint64_t start_lines = self->lines;
 
@@ -1362,7 +1354,8 @@ int _tokenize_helper(parser_t *self, size_t nrows, int all) {
         if (!all && self->lines - start_lines >= nrows) break;
 
         if (self->datapos == self->datalen) {
-            status = parser_buffer_bytes(self, self->chunksize);
+            status = parser_buffer_bytes(self, self->chunksize,
+                                         encoding_errors);
 
             if (status == REACHED_EOF) {
                 // close out last line
@@ -1395,13 +1388,13 @@ int _tokenize_helper(parser_t *self, size_t nrows, int all) {
     return status;
 }
 
-int tokenize_nrows(parser_t *self, size_t nrows) {
-    int status = _tokenize_helper(self, nrows, 0);
+int tokenize_nrows(parser_t *self, size_t nrows, const char *encoding_errors) {
+    int status = _tokenize_helper(self, nrows, 0, encoding_errors);
     return status;
 }
 
-int tokenize_all_rows(parser_t *self) {
-    int status = _tokenize_helper(self, -1, 1);
+int tokenize_all_rows(parser_t *self, const char *encoding_errors) {
+    int status = _tokenize_helper(self, -1, 1, encoding_errors);
     return status;
 }
 
@@ -1738,7 +1731,7 @@ double precise_xstrtod(const char *str, char **endptr, char decimal,
         // Process string of digits.
         num_digits = 0;
         n = 0;
-        while (isdigit_ascii(*p)) {
+        while (num_digits < max_digits && isdigit_ascii(*p)) {
             n = n * 10 + (*p - '0');
             num_digits++;
             p++;
@@ -1759,10 +1752,13 @@ double precise_xstrtod(const char *str, char **endptr, char decimal,
     } else if (exponent > 0) {
         number *= e[exponent];
     } else if (exponent < -308) {  // Subnormal
-        if (exponent < -616)       // Prevent invalid array access.
+        if (exponent < -616) {  // Prevent invalid array access.
             number = 0.;
-        number /= e[-308 - exponent];
-        number /= e[308];
+        } else {
+            number /= e[-308 - exponent];
+            number /= e[308];
+        }
+
     } else {
         number /= e[-exponent];
     }
@@ -1778,20 +1774,95 @@ double precise_xstrtod(const char *str, char **endptr, char decimal,
     return number;
 }
 
+/* copy a decimal number string with `decimal`, `tsep` as decimal point
+   and thousands separator to an equivalent c-locale decimal string (striping
+   `tsep`, replacing `decimal` with '.'). The returned memory should be free-d
+   with a call to `free`.
+*/
+
+char* _str_copy_decimal_str_c(const char *s, char **endpos, char decimal,
+                              char tsep) {
+    const char *p = s;
+    size_t length = strlen(s);
+    char *s_copy = malloc(length + 1);
+    char *dst = s_copy;
+    // Skip leading whitespace.
+    while (isspace_ascii(*p)) p++;
+    // Copy Leading sign
+    if (*p == '+' || *p == '-') {
+        *dst++ = *p++;
+    }
+    // Copy integer part dropping `tsep`
+    while (isdigit_ascii(*p)) {
+        *dst++ = *p++;
+        p += (tsep != '\0' && *p == tsep);
+    }
+    // Replace `decimal` with '.'
+    if (*p == decimal) {
+       *dst++ = '.';
+       p++;
+    }
+    // Copy fractional part after decimal (if any)
+    while (isdigit_ascii(*p)) {
+       *dst++ = *p++;
+    }
+    // Copy exponent if any
+    if (toupper_ascii(*p) == toupper_ascii('E')) {
+       *dst++ = *p++;
+       // Copy leading exponent sign (if any)
+       if (*p == '+' || *p == '-') {
+           *dst++ = *p++;
+       }
+       // Copy exponent digits
+       while (isdigit_ascii(*p)) {
+           *dst++ = *p++;
+       }
+    }
+    *dst++ = '\0';  // terminate
+    if (endpos != NULL)
+        *endpos = (char *)p;
+    return s_copy;
+}
+
+
 double round_trip(const char *p, char **q, char decimal, char sci, char tsep,
                   int skip_trailing, int *error, int *maybe_int) {
+    // 'normalize' representation to C-locale; replace decimal with '.' and
+    // remove t(housand)sep.
+    char *endptr;
+    char *pc = _str_copy_decimal_str_c(p, &endptr, decimal, tsep);
     // This is called from a nogil block in parsers.pyx
     // so need to explicitly get GIL before Python calls
     PyGILState_STATE gstate;
     gstate = PyGILState_Ensure();
-
-    double r = PyOS_string_to_double(p, q, 0);
+    char *endpc;
+    double r = PyOS_string_to_double(pc, &endpc, 0);
+    // PyOS_string_to_double needs to consume the whole string
+    if (endpc == pc + strlen(pc)) {
+        if (q != NULL) {
+           // report endptr from source string (p)
+            *q = endptr;
+        }
+    } else {
+        *error = -1;
+        if (q != NULL) {
+           // p and pc are different len due to tsep removal. Can't report
+           // how much it has consumed of p. Just rewind to beginning.
+           *q = (char *)p;  // TODO(willayd): this could be undefined behavior
+        }
+    }
     if (maybe_int != NULL) *maybe_int = 0;
     if (PyErr_Occurred() != NULL) *error = -1;
     else if (r == Py_HUGE_VAL) *error = (int)Py_HUGE_VAL;
     PyErr_Clear();
 
     PyGILState_Release(gstate);
+    free(pc);
+    if (skip_trailing && q != NULL && *q != p) {
+        while (isspace_ascii(**q)) {
+            (*q)++;
+        }
+    }
     return r;
 }
 
@@ -1810,7 +1881,7 @@ int uint64_conflict(uint_state *self) {
 
 int64_t str_to_int64(const char *p_item, int64_t int_min, int64_t int_max,
                      int *error, char tsep) {
-    const char *p = (const char *)p_item;
+    const char *p = p_item;
     int isneg = 0;
     int64_t number = 0;
     int d;
@@ -1930,7 +2001,7 @@ int64_t str_to_int64(const char *p_item, int64_t int_min, int64_t int_max,
 
 uint64_t str_to_uint64(uint_state *state, const char *p_item, int64_t int_max,
                        uint64_t uint_max, int *error, char tsep) {
-    const char *p = (const char *)p_item;
+    const char *p = p_item;
     uint64_t pre_max = uint_max / 10;
     int dig_pre_max = uint_max % 10;
     uint64_t number = 0;
