@@ -29,7 +29,7 @@ Portions of code from MODP_ASCII - Ascii transformations (upper/lower, etc)
 https://github.com/client9/stringencoders
 Copyright (c) 2007  Nick Galbreath -- nickg [at] modp [dot] com. All rights reserved.
 
-Numeric decoder derived from from TCL library
+Numeric decoder derived from TCL library
 https://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
 * Copyright (c) 1988-1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
@@ -38,10 +38,12 @@ https://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
 #include "version.h"
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
+#define PY_ARRAY_UNIQUE_SYMBOL UJSON_NUMPY
+#include "numpy/arrayobject.h"
 
 /* objToJSON */
 PyObject *objToJSON(PyObject *self, PyObject *args, PyObject *kwargs);
-void initObjToJSON(void);
+void *initObjToJSON(void);
 
 /* JSONToObj */
 PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs);
@@ -73,7 +75,7 @@ static PyModuleDef moduledef = {
 
 
 PyMODINIT_FUNC PyInit_json(void) {
-  initObjToJSON();  // TODO: clean up, maybe via tp_free?
+  import_array()
+  initObjToJSON();  // TODO(username): clean up, maybe via tp_free?
   return PyModuleDef_Init(&moduledef);
-
 }
