@@ -20,7 +20,11 @@ from pandas import (
     timedelta_range,
 )
 import pandas._testing as tm
-from pandas.core.arrays import IntervalArray
+from pandas.core.arrays import (
+    BooleanArray,
+    IntervalArray,
+)
+from pandas.tests.arithmetic.common import get_upcast_box
 
 
 @pytest.fixture(
@@ -28,16 +32,16 @@ from pandas.core.arrays import IntervalArray
         (Index([0, 2, 4, 4]), Index([1, 3, 5, 8])),
         (Index([0.0, 1.0, 2.0, np.nan]), Index([1.0, 2.0, 3.0, np.nan])),
         (
-            timedelta_range("0 days", periods=3).insert(4, pd.NaT),
-            timedelta_range("1 day", periods=3).insert(4, pd.NaT),
+            timedelta_range("0 days", periods=3).insert(3, pd.NaT),
+            timedelta_range("1 day", periods=3).insert(3, pd.NaT),
         ),
         (
-            date_range("20170101", periods=3).insert(4, pd.NaT),
-            date_range("20170102", periods=3).insert(4, pd.NaT),
+            date_range("20170101", periods=3).insert(3, pd.NaT),
+            date_range("20170102", periods=3).insert(3, pd.NaT),
         ),
         (
-            date_range("20170101", periods=3, tz="US/Eastern").insert(4, pd.NaT),
-            date_range("20170102", periods=3, tz="US/Eastern").insert(4, pd.NaT),
+            date_range("20170101", periods=3, tz="US/Eastern").insert(3, pd.NaT),
+            date_range("20170102", periods=3, tz="US/Eastern").insert(3, pd.NaT),
         ),
     ],
     ids=lambda x: str(x[0].dtype),
@@ -129,18 +133,37 @@ class TestComparison:
         expected = self.elementwise_comparison(op, interval_array, other)
         tm.assert_numpy_array_equal(result, expected)
 
-    def test_compare_scalar_na(self, op, interval_array, nulls_fixture, request):
-        result = op(interval_array, nulls_fixture)
-        expected = self.elementwise_comparison(op, interval_array, nulls_fixture)
+    def test_compare_scalar_na(
+        self, op, interval_array, nulls_fixture, box_with_array, request
+    ):
+        box = box_with_array
 
-        if nulls_fixture is pd.NA and interval_array.dtype.subtype != "int64":
-            mark = pytest.mark.xfail(
-                raises=AssertionError,
-                reason="broken for non-integer IntervalArray; see GH 31882",
-            )
-            request.node.add_marker(mark)
+        if box is pd.DataFrame:
+            if interval_array.dtype.subtype.kind not in "iuf":
+                mark = pytest.mark.xfail(
+                    reason="raises on DataFrame.transpose (would be fixed by EA2D)"
+                )
+                request.node.add_marker(mark)
 
-        tm.assert_numpy_array_equal(result, expected)
+        obj = tm.box_expected(interval_array, box)
+        result = op(obj, nulls_fixture)
+
+        if nulls_fixture is pd.NA:
+            # GH#31882
+            exp = np.ones(interval_array.shape, dtype=bool)
+            expected = BooleanArray(exp, exp)
+        else:
+            expected = self.elementwise_comparison(op, interval_array, nulls_fixture)
+
+        if not (box is Index and nulls_fixture is pd.NA):
+            # don't cast expected from BooleanArray to ndarray[object]
+            xbox = get_upcast_box(obj, nulls_fixture, True)
+            expected = tm.box_expected(expected, xbox)
+
+        tm.assert_equal(result, expected)
+
+        rev = op(nulls_fixture, obj)
+        tm.assert_equal(rev, expected)
 
     @pytest.mark.parametrize(
         "other",
@@ -214,17 +237,12 @@ class TestComparison:
         expected = self.elementwise_comparison(op, interval_array, other)
         tm.assert_numpy_array_equal(result, expected)
 
-    def test_compare_list_like_nan(self, op, interval_array, nulls_fixture, request):
+    def test_compare_list_like_nan(self, op, interval_array, nulls_fixture):
         other = [nulls_fixture] * 4
         result = op(interval_array, other)
         expected = self.elementwise_comparison(op, interval_array, other)
 
-        if nulls_fixture is pd.NA and interval_array.dtype.subtype != "i8":
-            reason = "broken for non-integer IntervalArray; see GH 31882"
-            mark = pytest.mark.xfail(raises=AssertionError, reason=reason)
-            request.node.add_marker(mark)
-
-        tm.assert_numpy_array_equal(result, expected)
+        tm.assert_equal(result, expected)
 
     @pytest.mark.parametrize(
         "other",
