@@ -27,7 +27,7 @@ from pandas.tests.io.pytables.common import (
 
 from pandas.io.pytables import Term
 
-pytestmark = pytest.mark.single
+pytestmark = pytest.mark.single_cpu
 
 
 def test_select_columns_in_where(setup_path):
@@ -188,12 +188,12 @@ def test_select_dtypes(setup_path):
         _maybe_remove(store, "df")
         store.append("df", df, data_columns=True)
 
-        expected = df[df.boolv == True].reindex(columns=["A", "boolv"])  # noqa
+        expected = df[df.boolv == True].reindex(columns=["A", "boolv"])  # noqa:E712
         for v in [True, "true", 1]:
             result = store.select("df", f"boolv == {v}", columns=["A", "boolv"])
             tm.assert_frame_equal(expected, result)
 
-        expected = df[df.boolv == False].reindex(columns=["A", "boolv"])  # noqa
+        expected = df[df.boolv == False].reindex(columns=["A", "boolv"])  # noqa:E712
         for v in [False, "false", 0]:
             result = store.select("df", f"boolv == {v}", columns=["A", "boolv"])
             tm.assert_frame_equal(expected, result)
@@ -265,7 +265,7 @@ def test_select_dtypes(setup_path):
         expected = df[df["A"] > 0]
 
         store.append("df", df, data_columns=True)
-        np_zero = np.float64(0)  # noqa
+        np_zero = np.float64(0)  # noqa:F841
         result = store.select("df", where=["A>np_zero"])
         tm.assert_frame_equal(expected, result)
 
@@ -659,7 +659,7 @@ def test_frame_select_complex(setup_path):
         tm.assert_frame_equal(result, expected)
 
 
-def test_frame_select_complex2(setup_path):
+def test_frame_select_complex2():
 
     with ensure_clean_path(["params.hdf", "hist.hdf"]) as paths:
 
@@ -683,17 +683,17 @@ def test_frame_select_complex2(setup_path):
         expected = read_hdf(hh, "df", where="l1=[2, 3, 4]")
 
         # scope with list like
-        l = selection.index.tolist()  # noqa
+        l0 = selection.index.tolist()  # noqa:F841
         store = HDFStore(hh)
-        result = store.select("df", where="l1=l")
+        result = store.select("df", where="l1=l0")
         tm.assert_frame_equal(result, expected)
         store.close()
 
-        result = read_hdf(hh, "df", where="l1=l")
+        result = read_hdf(hh, "df", where="l1=l0")
         tm.assert_frame_equal(result, expected)
 
         # index
-        index = selection.index  # noqa
+        index = selection.index  # noqa:F841
         result = read_hdf(hh, "df", where="l1=index")
         tm.assert_frame_equal(result, expected)
 
@@ -928,7 +928,7 @@ def test_query_compare_column_type(setup_path):
     with ensure_clean_store(setup_path) as store:
         store.append("test", df, format="table", data_columns=True)
 
-        ts = Timestamp("2014-01-01")  # noqa
+        ts = Timestamp("2014-01-01")  # noqa:F841
         result = store.select("test", where="real_date > ts")
         expected = df.loc[[1], :]
         tm.assert_frame_equal(expected, result)
