@@ -17,39 +17,39 @@ from __future__ import annotations
 
 import argparse
 import doctest
-import glob
 import importlib
+import io
 import json
 import os
+import pathlib
 import subprocess
 import sys
 import tempfile
 
-try:
-    from io import StringIO
-except ImportError:
-    from cStringIO import StringIO
+import matplotlib
+import matplotlib.pyplot as plt
+import numpy
+from numpydoc.docscrape import get_doc_object
+from numpydoc.validate import (
+    Validator,
+    validate,
+)
 
-# Template backend makes matplotlib to not plot anything. This is useful
-# to avoid that plot windows are open from the doctests while running the
-# script. Setting here before matplotlib is loaded.
-# We don't warn for the number of open plots, as none is actually being opened
-os.environ["MPLBACKEND"] = "Template"
-import matplotlib  # isort:skip
-
-matplotlib.rc("figure", max_open_warning=10000)
-
-import numpy  # isort:skip
-
-BASE_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-
-sys.path.insert(0, os.path.join(BASE_PATH))
-import pandas  # isort:skip
-
-sys.path.insert(1, os.path.join(BASE_PATH, "doc", "sphinxext"))
-from numpydoc.validate import validate, Docstring  # isort:skip
+import pandas
 
+# With template backend, matplotlib plots nothing
+matplotlib.use("template")
 
+# Styler methods are Jinja2 objects who's docstrings we don't own.
+IGNORE_VALIDATION = {
+    "Styler.env",
+    "Styler.template_html",
+    "Styler.template_html_style",
+    "Styler.template_html_table",
+    "Styler.template_latex",
+    "Styler.template_string",
+    "Styler.loader",
+}
 PRIVATE_CLASSES = ["NDFrame", "IndexOpsMixin"]
 ERROR_MSGS = {
     "GL04": "Private classes ({mentioned_private_classes}) should not be "
@@ -130,6 +130,8 @@ def get_api_items(api_doc_fd):
                 position = None
                 continue
             item = line.strip()
+            if item in IGNORE_VALIDATION:
+                continue
             func = importlib.import_module(current_module)
             for part in item.split("."):
                 func = getattr(func, part)
@@ -144,7 +146,17 @@ def get_api_items(api_doc_fd):
         previous_line = line
 
 
-class PandasDocstring(Docstring):
+class PandasDocstring(Validator):
+    def __init__(self, func_name: str, doc_obj=None) -> None:
+        self.func_name = func_name
+        if doc_obj is None:
+            doc_obj = get_doc_object(Validator._load_obj(func_name))
+        super().__init__(doc_obj)
+
+    @property
+    def name(self):
+        return self.func_name
+
     @property
     def mentioned_private_classes(self):
         return [klass for klass in PRIVATE_CLASSES if klass in self.raw_doc]
@@ -156,10 +168,23 @@ class PandasDocstring(Docstring):
         runner = doctest.DocTestRunner(optionflags=flags)
         context = {"np": numpy, "pd": pandas}
         error_msgs = ""
+        current_dir = set(os.listdir())
         for test in finder.find(self.raw_doc, self.name, globs=context):
-            f = StringIO()
+            f = io.StringIO()
             runner.run(test, out=f.write)
             error_msgs += f.getvalue()
+        leftovers = set(os.listdir()).difference(current_dir)
+        if leftovers:
+            for leftover in leftovers:
+                path = pathlib.Path(leftover).resolve()
+                if path.is_dir():
+                    path.rmdir()
+                elif path.is_file():
+                    path.unlink(missing_ok=True)
+            raise Exception(
+                f"The following files were leftover from the doctest: "
+                f"{leftovers}. Please use # doctest: +SKIP"
+            )
         return error_msgs
 
     @property
@@ -215,8 +240,11 @@ def pandas_validate(func_name: str):
     dict
         Information about the docstring and the errors found.
     """
-    doc = PandasDocstring(func_name)
-    result = validate(func_name)
+    func_obj = Validator._load_obj(func_name)
+    # Some objects are instances, e.g. IndexSlice, which numpydoc can't validate
+    doc_obj = get_doc_object(func_obj, doc=func_obj.__doc__)
+    doc = PandasDocstring(func_name, doc_obj)
+    result = validate(doc_obj)
 
     mentioned_errs = doc.mentioned_private_classes
     if mentioned_errs:
@@ -263,6 +291,7 @@ def pandas_validate(func_name: str):
     if doc.non_hyphenated_array_like():
         result["errors"].append(pandas_error("GL05"))
 
+    plt.close("all")
     return result
 
 
@@ -288,13 +317,14 @@ def validate_all(prefix, ignore_deprecated=False):
     result = {}
     seen = {}
 
-    api_doc_fnames = os.path.join(BASE_PATH, "doc", "source", "reference", "*.rst")
+    base_path = pathlib.Path(__file__).parent.parent
+    api_doc_fnames = pathlib.Path(base_path, "doc", "source", "reference")
     api_items = []
-    for api_doc_fname in glob.glob(api_doc_fnames):
+    for api_doc_fname in api_doc_fnames.glob("*.rst"):
         with open(api_doc_fname) as f:
             api_items += list(get_api_items(f))
 
-    for func_name, func_obj, section, subsection in api_items:
+    for func_name, _, section, subsection in api_items:
         if prefix and not func_name.startswith(prefix):
             continue
         doc_info = pandas_validate(func_name)
