@@ -5,4 +5,4 @@
 # https://github.com/pytest-dev/pytest/issues/1075
 export PYTHONHASHSEED=$(python -c 'import random; print(random.randint(1, 4294967295))')
 
-pytest pandas --skip-slow --skip-network --skip-db -m "not single" -n 4 -r sxX "$@"
+pytest pandas --skip-slow --skip-network --skip-db -m "not single_cpu" -n 4 -r sxX "$@"
